#include <glib.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <tree.h>
#include <parser.h>

xmlNodePtr colors[12];
xmlNodePtr editor,interface,render;
extern char *fontchar;
extern xmlDocPtr conftree;

GdkColor *str2color(char *str)
{
  GdkColor *color=(GdkColor *)malloc(sizeof(GdkColor));
  char **tab;
  
  if (str==NULL)
    return NULL;
  tab=g_strsplit(str,":",3);
  color->pixel=0;
  color->red=atoi(tab[0]);
  color->green=atoi(tab[1]);
  color->blue=atoi(tab[2]);
  
  return color;
}

char *color2str(GdkColor *color)
{
  char *temp;

  if (color==NULL) return NULL;
  temp=g_strdup_printf("%d:%d:%d",color->red,color->green,color->blue);

  return temp;
}

GdkColor *xmlconf_get_color( int index)
{
  return str2color(g_strdup(xmlGetProp(colors[index],"value")));
}

void xmlconf_set_color( GdkColor *color, int index)
{
  xmlSetProp(colors[index],"value",color2str(color));
}

char *xmlconf_get_font(void)
{
  return g_strdup(xmlGetProp(editor,"font"));
}

void xmlconf_set_font(char *font)
{
  xmlSetProp(conftree->root->childs->next,"font",font);
}

void xmlconf_set_render(int a, int b, int c)
{
  if (a!=-1) xmlSetProp(render,"filenpic",g_strdup_printf("%d",a));
  if (b!=-1) xmlSetProp(render,"format",g_strdup_printf("%d",b));
  if (c!=-1) xmlSetProp(render,"taille",g_strdup_printf("%d",c));
}

int xmlconf_get_render(int num)
{
  switch (num)
    {
    case 0:
      return atoi(xmlGetProp(render,"filenpic"));
    case 1:
      return atoi(xmlGetProp(render,"format"));
    case 2:
      return atoi(xmlGetProp(render,"taille"));
    default:
      return -1;
    }
}

xmlDocPtr initconf(void)
{
  int i;
  xmlDocPtr doc;
  
  if ((doc=xmlParseFile(g_strconcat(g_get_home_dir(),
				    "/.modlight",NULL)))==NULL)
    if ((doc=xmlParseFile("/etc/modlight.conf"))==NULL)
      if ((doc=xmlParseFile("modlight.conf"))==NULL)
    	{
	  xmlNodePtr tree,subtree;

	  doc=xmlNewDoc("1.0");
	  doc->root=xmlNewDocNode(doc,NULL,"CONFIG",NULL);
	  xmlSetProp(doc->root,"file",g_strconcat(g_get_home_dir(),"/.modlight",NULL));
	  interface=xmlNewChild(doc->root,NULL,"INTERFACE",NULL);
	  editor=xmlNewChild(doc->root,NULL,"EDITOR",NULL);
	  xmlSetProp(editor,"font",
		     "-adobe-helvetica-bold-r-narrow--17-0-75-75-p-74-iso8859-1");
	  colors[0]=xmlNewChild(editor,NULL,"default",NULL);
	  xmlSetProp(colors[0],"value","0:0:0");
	  colors[1]=xmlNewChild(editor,NULL,"comment",NULL);
	  xmlSetProp(colors[1],"value","65535:0:0");
	  colors[2]=xmlNewChild(editor,NULL,"include",NULL);
	  xmlSetProp(colors[2],"value","0:0:65535");
	  colors[3]=xmlNewChild(editor,NULL,"declare",NULL);
	  xmlSetProp(colors[3],"value","35328:42472:59904");
	  colors[4]=xmlNewChild(editor,NULL,"base",NULL);
	  xmlSetProp(colors[4],"value","0:65535:65535");
	  colors[5]=xmlNewChild(editor,NULL,"objet",NULL);
	  xmlSetProp(colors[5],"value","0:65535:0");
	  colors[6]=xmlNewChild(editor,NULL,"operation",NULL);
	  xmlSetProp(colors[6],"value","65535:0:65535");
	  colors[7]=xmlNewChild(editor,NULL,"texture",NULL);
	  xmlSetProp(colors[7],"value","65535:65535:0");
	  colors[8]=xmlNewChild(editor,NULL,"transformation",NULL);
	  xmlSetProp(colors[8],"value","63232:42240:0512");
	  colors[9]=xmlNewChild(editor,NULL,"constant",NULL);
	  xmlSetProp(colors[9],"value","39321:39321:39321");
	  colors[10]=xmlNewChild(editor,NULL,"operator",NULL);
	  xmlSetProp(colors[10],"value","65535:0:65535");
	  colors[11]=xmlNewChild(editor,NULL,"separator",NULL);
	  xmlSetProp(colors[11],"value","64521:31488:37120");
	  render=xmlNewChild(doc->root,NULL,"RENDER",NULL);
	  xmlSetProp(render,"filenpic","0");
	  xmlSetProp(render,"format","0");
	  xmlSetProp(render,"taille","0");
	  return doc;
	}
  interface=doc->root->childs;
  editor=interface->next;
  render=editor->next;
  colors[0]=editor->childs;
  for(i=1;i<12;i++)
    colors[i]=colors[i-1]->next;
  
  return doc;
}     

int saveconf(xmlDocPtr doc)
{
  xmlSetCompressMode(0);
  xmlSaveFile(xmlGetProp(xmlDocGetRootElement(doc),"file"),doc);
  return 0;
}




