/* start condition definitions */
%x STRING

/* pattern definitions */
D			[0-9]
L			[a-zA-Z_]
H			[a-fA-F0-9]
E			[Ee][+-]?{D}+
FS			(f|F|l|L)
IS			(u|U|l|L)*

%option fast always-interactive noyywrap prefix="jgc"
%option noyy_scan_buffer noyy_scan_bytes noyy_scan_string

%{

#define FALSE 0
#define TRUE 1

#define YY_INPUT(buf,result,max_size) \
{ \
  int c = next_char(); \
  result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
}

#define YY_DECL int lex ()
#define YY_USER_ACTION position += prev_leng; prev_leng = yyleng;


static void (*new_token)(int id, int precedence, char *text, int length, int position, int block_id, int nesting, int block_type);

static int (*next_char)();

%}

%%

%{
int position = 0, prev_leng = 0;
%}


 /* patterns */
"/*"           new_token (-1,  0, yytext, yyleng, position, 0,  FALSE, 1);
"*/"           new_token (-1, 0, yytext, yyleng, position, 0, 0, 0);
\"             new_token (-1,  0, yytext, yyleng, position, 1,  FALSE, 1); BEGIN(STRING);

<STRING>{
\\\n         /* continued string */
\n           new_token (-1, 0, yytext, yyleng, position, 1, 0, 0); BEGIN(INITIAL);
\\.          |
\\[0-7]{1,3} new_token (0,  0, yytext, yyleng, position, -1, 0, 0);
\"           new_token (-1, 0, yytext, yyleng, position, 1, 0, 0); BEGIN(INITIAL);
.
}


^[ \t]*#[ \t]*include[ \t]*((\"[^<>\n]*\"?)|("<"[^\"\n]*">"?)) new_token (1,  0, yytext, yyleng, position, -1, 0, 0);

^[ \t]*#[ \t]*(declare) new_token (2,  0, yytext, yyleng, position, -1, 0, 0);


"background"   |
"camera"       |
"light_source"          new_token (3,  0, yytext, yyleng, position, -1, 0, 0); 
	
"box"      |
"cone"     |
"cylinder" |
"sphere"                new_token (4,  0, yytext, yyleng, position, -1, 0, 0);

"merge"        |
"union"        |
"difference"   |
"intersection"          new_token (5,  0, yytext, yyleng, position, -1, 0, 0);

"pigment"   |
"texture"   |
"color"                 new_token (6,  0, yytext, yyleng, position, -1, 0, 0); 
	
"*x"        |
"*y"        |
"*z"        | 
"scale"     |
"rotate"    |
"translate"             new_token (7,  0, yytext, yyleng, position, -1, 0, 0);

{L}({L}|{D})*		/* identifier ignore */

0[xX]{H}+{IS}?		|
0{D}+{IS}?		|
{D}+{IS}?		|
'(\\.|[^\\'])+'		|

{D}+{E}{FS}?		|
{D}*"."{D}+({E})?{FS}?	|
{D}+"."{D}*({E})?{FS}?	new_token (8,  0, yytext, yyleng, position, -1, 0, 0);	


"-" |
"+" |
"*" |
"/" |
"%" |
"<" |
">" |
"="                     new_token (9,  0, yytext, yyleng, position, -1, 0, 0);

"{" |
"}" |
"(" |
")" |
"[" |
"]"                     new_token (10,  0, yytext, yyleng, position, -1, 0, 0); 

"."
","
":"
";" 
	
[ \t\v\n\f]		
.	     /* ignore bad characters */

%%


const char *
get_name ()

{
  const char *name = NULL;

  return name;
}


const char **
get_token_names ()

{
  static const char *token_names[] = {"escape_char", "include", "declare", "base", "objet", "operation", "texture", "transformation", "constant", "operator", "separator", NULL};

  return token_names;
}

const char **
get_block_names ()

{
  static const char *block_names[] = {"comment", "string", NULL};

  return block_names;
}


int
get_token_no (const char *name)

{
  int number = -1;

  if (strcmp("escape_char", name) == 0) {
    return 0;
  } else if (strcmp("include", name) == 0) {
    return 1;
  } else if (strcmp("declare", name) == 0) {
    return 2;
  } else if (strcmp("base", name) == 0) {
    return 3;
  } else if (strcmp("objet", name) == 0) {
    return 4;
  } else if (strcmp("operation", name) == 0) {
    return 5;
  } else if (strcmp("texture", name) == 0) {
    return 6;
  } else if (strcmp("transformation", name) == 0) {
    return 7;
  } else if (strcmp("constant", name) == 0) {
    return 8;
  } else if (strcmp("operator", name) == 0) {
    return 9;
  } else if (strcmp("separator", name) == 0) {
    return 10;
  }

  return number;
}


int
get_block_no (const char *name)

{
  int number = -1;

  if (strcmp("comment", name) == 0) {
    return 0;
  } else if (strcmp("string", name) == 0) {
    return 1;
  }

  return number;
}

void
token_func (void (*func)(int id, int precedence, char *text, int length, int position, int block_id, int nesting, int block_type))
{
  new_token = func;
}

void
char_func (int (*func)())
{
  next_char = func;
}

int
get_state ()
{
  return YY_START;
}

void
set_state (int state)
{
  BEGIN (state);
}