#include "headers.h"
#include "matrix.h"


VECT3D NormalizeVect(VECT3D u)
{
  float norme;

  norme = sqrt(u.x * u.x + u.y * u.y + u.z * u.z);
  u.x = u.x / norme;
  u.y = u.y / norme;
  u.z = u.z / norme;
  return (u);
}


float DotProduct(VECT3D u,VECT3D v)
{
  return(u.x*v.x + u.y*v.y + u.z*v.z);
}


VECT3D CrossProduct(VECT3D u,VECT3D v)
{
  VECT3D res;
  res.x = u.y * v.z - u.z * v.y;
  res.y = u.z * v.x - u.x * v.z;
  res.z = u.x * v.y - u.y * v.x;

  return(res);
}


void CalculateNormal(T_obj *obj)
{
  VECT3D	v1,v2;
  T_face	*face;
  int		i;

  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      face = ReadTriangleOfMesh(obj->prop_base->list_vertex, i);
      v1.x = face->B->x - face->A->x;
      v1.y = face->B->y - face->A->y;
      v1.z = face->B->z - face->A->z;
      
      v2.x = face->C->x - face->A->x;
      v2.y = face->C->y - face->A->y;
      v2.z = face->C->z - face->A->z;
      obj->prop_base->normal[i] = CrossProduct(v1, v2);
      obj->prop_base->normal[i] = NormalizeVect(obj->prop_base->normal[i]);
    }
}


float SqrNorme2D(VECT2D u, VECT2D v)
{
  return (pow(u.x-v.x, 2) +  pow(u.y-v.y, 2));
}

float NormeVect2D(VECT2D u)
{
  return (sqrt(pow(u.x, 2) + pow(u.y, 2)));
}

float ComputeAngle(VECT2D u)
{
  VECT2D tmp;

  if (u.y == 0)
    {
      if (u.x > 0)
	return 0;
      else
	return 180;
    }
  if (u.x == 0)
    {
      if (u.y > 0)
	return 90;
      else
	return 270;
    }
  tmp.x = u.x;
  tmp.y = 0;
  if (u.y > 0)
    {
      if (u.x > 0)	/* first quadran*/
	return acos(NormeVect2D(tmp)/NormeVect2D(u));
      if (u.x < 0)	/* second quadran */
	return 180 - acos(NormeVect2D(tmp)/NormeVect2D(u));
    }
  else
    {
      if (u.x < 0)	/* third quadran */
	return 180 + acos(NormeVect2D(tmp)/NormeVect2D(u));
      if (u.x > 0)
	return 360 - acos(NormeVect2D(tmp)/NormeVect2D(u));
    }
  return 0;
}
