#include "headers.h"
#include "core_plugins/src/core_sor.h"

GSList *AddTriangleToMesh(GSList *list, T_face *face)
{
  list = g_slist_append(list, (gpointer)face);
  return list;
}

T_face *ReadTriangleOfMesh(GSList *list, int index_face)
{
  T_face	*face;
  GSList	*current_face;

  current_face = g_slist_nth(list, index_face);
  face = (T_face *)current_face->data;
  return face;
}

T_face *make_face(VECT3D *A, VECT3D *B, VECT3D *C)
{
  T_face *face;
  
  face = (T_face *)malloc(sizeof(T_face));
  face->A = (VECT3D *)malloc(sizeof(VECT3D));
  face->B = (VECT3D *)malloc(sizeof(VECT3D));
  face->C = (VECT3D *)malloc(sizeof(VECT3D));
  memcpy(face->A, A, sizeof(VECT3D));
  memcpy(face->B, B, sizeof(VECT3D));
  memcpy(face->C, C, sizeof(VECT3D));
  return face;
}

T_obj *make_MeshTriangleBox(VECT3D *MinCorner, VECT3D *MaxCorner)
{
  T_obj			*obj;
  VECT3D		*vertex[8];
  T_face		*face;
  int			i;

  obj = (T_obj *)malloc(sizeof(T_obj));
  obj->name = "box";
  obj->type_obj = OBJ_BOX;
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->prop->box.MinCorner = MinCorner;
  obj->prop->box.MaxCorner = MaxCorner;
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));
  obj->prop_base->nb_face = 12;
  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->list_vertex = NULL;
  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;
  
  for (i = 0; i < 8; i++)
    vertex[i] = (VECT3D *)malloc(sizeof(VECT3D));
  vertex[0]->x = MinCorner->x;
  vertex[0]->y = MinCorner->y;
  vertex[0]->z = MinCorner->z;
  vertex[1]->x = MaxCorner->x;
  vertex[1]->y = MinCorner->y;
  vertex[1]->z = MinCorner->z;
  vertex[2]->x = MinCorner->x;
  vertex[2]->y = MaxCorner->y;
  vertex[2]->z = MinCorner->z;
  vertex[3]->x = MaxCorner->x;
  vertex[3]->y = MaxCorner->y;
  vertex[3]->z = MinCorner->z;
  vertex[4]->x = MinCorner->x;
  vertex[4]->y = MinCorner->y;
  vertex[4]->z = MaxCorner->z;
  vertex[5]->x = MaxCorner->x;
  vertex[5]->y = MinCorner->y;
  vertex[5]->z = MaxCorner->z;
  vertex[6]->x = MinCorner->x;
  vertex[6]->y = MaxCorner->y;
  vertex[6]->z = MaxCorner->z;
  vertex[7]->x = MaxCorner->x;
  vertex[7]->y = MaxCorner->y;
  vertex[7]->z = MaxCorner->z;

  face = make_face(vertex[2], vertex[0], vertex[4]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[4], vertex[6], vertex[2]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[3], vertex[2], vertex[6]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[6], vertex[7], vertex[3]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[0], vertex[1], vertex[5]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[5], vertex[4], vertex[0]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[1], vertex[3], vertex[7]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[7], vertex[5], vertex[1]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[4], vertex[5], vertex[7]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[7], vertex[6], vertex[4]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[0], vertex[2], vertex[3]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(vertex[3], vertex[1], vertex[0]); 
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;
      obj->prop_base->color[i].y = 0.5;
      obj->prop_base->color[i].z = 0.5;
    }
  obj->prop_base->is_selected = 0;
  return obj;
}

T_obj *make_MeshTriangleSphere(VECT3D *center, float radius)
{
  int		i;
  int		j;
  VECT3D	*A;
  VECT3D	*B;
  VECT3D	*C;
  VECT3D	*D;
  T_face	*face;
  T_obj		*obj;
  
  A = (VECT3D *)malloc(sizeof(VECT3D));
  B = (VECT3D *)malloc(sizeof(VECT3D));
  C = (VECT3D *)malloc(sizeof(VECT3D));
  D = (VECT3D *)malloc(sizeof(VECT3D));

  obj = (T_obj *)malloc(sizeof(T_obj));
  /* basics properties */
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));
  obj->name = "sphere";
  obj->type_obj = OBJ_SPHERE;
  obj->prop->sphere.center = center;
  obj->prop->sphere.radius = radius;
  obj->prop->sphere.detail_level = 25;
  obj->prop_base->list_vertex = NULL;
  obj->prop_base->nb_face = (obj->prop->sphere.detail_level*
			    (obj->prop->sphere.detail_level - 2) * 2 +
			    (obj->prop->sphere.detail_level) * 2);
  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D)*obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D)*obj->prop_base->nb_face);

  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;

  /* north pole */
  A->x = obj->prop->sphere.center->x;
  A->y = obj->prop->sphere.center->y + obj->prop->sphere.radius;
  A->z = obj->prop->sphere.center->z;
  
  for (i = 0; i < obj->prop->sphere.detail_level; i++)
    {
      B->x = obj->prop->sphere.center->x + obj->prop->sphere.radius * 
	cos(i * 2 * M_PI / obj->prop->sphere.detail_level) *
	sin(M_PI / obj->prop->sphere.detail_level);
      B->y = obj->prop->sphere.center->y + obj->prop->sphere.radius * 
	cos(M_PI / obj->prop->sphere.detail_level);
      B->z = obj->prop->sphere.center->z + obj->prop->sphere.radius * 
	sin(M_PI / obj->prop->sphere.detail_level) *
	sin(i * 2 * M_PI / obj->prop->sphere.detail_level);
      
      C->x = obj->prop->sphere.center->x + obj->prop->sphere.radius * 
	cos((i + 1) * 2 * M_PI / obj->prop->sphere.detail_level) *
	sin(M_PI / obj->prop->sphere.detail_level);
      C->y = obj->prop->sphere.center->y + obj->prop->sphere.radius * 
	cos(M_PI / obj->prop->sphere.detail_level);
      C->z = obj->prop->sphere.center->z + obj->prop->sphere.radius * 
	sin(M_PI / obj->prop->sphere.detail_level) *
	sin((i + 1) * 2 * M_PI / obj->prop->sphere.detail_level);
      face = make_face(B, A, C);
      obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						       face);
    }
  /*  beetween then north and south pole */
  for (i = 1; i < obj->prop->sphere.detail_level - 1; i++)
    for (j = 0; j < obj->prop->sphere.detail_level; j++)
      {
	A->x = obj->prop->sphere.center->x + obj->prop->sphere.radius *
	  cos(j * 2 * M_PI / obj->prop->sphere.detail_level) *
	  sin(i * M_PI / obj->prop->sphere.detail_level);
	A->y = obj->prop->sphere.center->y + obj->prop->sphere.radius *
	  cos(i * M_PI / obj->prop->sphere.detail_level);
	A->z = obj->prop->sphere.center->z + obj->prop->sphere.radius *
	  sin(i * M_PI / obj->prop->sphere.detail_level) *
	  sin(j * 2 * M_PI / obj->prop->sphere.detail_level);
	B->x = obj->prop->sphere.center->x + obj->prop->sphere.radius *
	  cos((j + 1) * 2 * M_PI / obj->prop->sphere.detail_level) *
	  sin(i * M_PI / obj->prop->sphere.detail_level);
	B->y = obj->prop->sphere.center->y + obj->prop->sphere.radius *
	  cos(i * M_PI / obj->prop->sphere.detail_level);
	B->z = obj->prop->sphere.center->z + obj->prop->sphere.radius *
	  sin(i * M_PI / obj->prop->sphere.detail_level) *
	  sin((j + 1) * 2 * M_PI / obj->prop->sphere.detail_level);
	C->x = obj->prop->sphere.center->x + obj->prop->sphere.radius *
	  cos(j * 2 * M_PI / obj->prop->sphere.detail_level) *
	  sin((i + 1) * M_PI / obj->prop->sphere.detail_level);
	C->y = obj->prop->sphere.center->y + obj->prop->sphere.radius *
	  cos((i + 1) * M_PI / obj->prop->sphere.detail_level);
	C->z = obj->prop->sphere.center->z + obj->prop->sphere.radius *
	  sin((i + 1) * M_PI / obj->prop->sphere.detail_level) *
	  sin(j * 2 * M_PI / obj->prop->sphere.detail_level);
	D->x = obj->prop->sphere.center->x + obj->prop->sphere.radius *
	  cos((j + 1) * 2 * M_PI / obj->prop->sphere.detail_level) *
	  sin((i + 1) * M_PI / obj->prop->sphere.detail_level);
	D->y = obj->prop->sphere.center->y + obj->prop->sphere.radius *
	  cos((i + 1) * M_PI / obj->prop->sphere.detail_level);
	D->z = obj->prop->sphere.center->z + obj->prop->sphere.radius *
	  sin((i + 1) * M_PI / obj->prop->sphere.detail_level) *
	  sin((j + 1) * 2 * M_PI / obj->prop->sphere.detail_level);

	face = make_face(A, B, C);
	obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						       face);
	face = make_face(C, B, D);
	obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
							face);
      }

  /* South pole */
  A->x = obj->prop->sphere.center->x;
  A->y = obj->prop->sphere.center->y - obj->prop->sphere.radius;
  A->z = obj->prop->sphere.center->z;
  
  for (i = 0; i < obj->prop->sphere.detail_level; i++)
    {
      B->x = obj->prop->sphere.center->x + obj->prop->sphere.radius * 
	cos(i * 2 * M_PI / obj->prop->sphere.detail_level) *
	sin(M_PI / obj->prop->sphere.detail_level);
      B->y = obj->prop->sphere.center->y - obj->prop->sphere.radius * 
	cos(M_PI / obj->prop->sphere.detail_level);
      B->z = obj->prop->sphere.center->z + obj->prop->sphere.radius * 
	sin(M_PI / obj->prop->sphere.detail_level) *
	sin(i * 2 * M_PI / obj->prop->sphere.detail_level);
      
      C->x = obj->prop->sphere.center->x + obj->prop->sphere.radius * 
	cos((i + 1) * 2 * M_PI / obj->prop->sphere.detail_level) *
	sin(M_PI / obj->prop->sphere.detail_level);
      C->y = obj->prop->sphere.center->y - obj->prop->sphere.radius * 
	cos(M_PI / obj->prop->sphere.detail_level);
      C->z = obj->prop->sphere.center->z + obj->prop->sphere.radius * 
	sin(M_PI / obj->prop->sphere.detail_level) *
	sin((i + 1) * 2 * M_PI / obj->prop->sphere.detail_level);

      face = make_face(A, B, C);
      obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						     face);
    }
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;	/* red <=> x */
      obj->prop_base->color[i].y = 0.5;	/* green <=> y */
      obj->prop_base->color[i].z = 0.5;	/* blue <=> z */
    }
  free(A);
  free(B);
  free(C);
  free(D);
  obj->prop_base->is_selected = 0;
  return obj;
}


T_obj *make_MeshTriangleCone(VECT3D *c1, float r1, int is_openC1, 
			     VECT3D *c2, float r2, int is_openC2,
			     int is_double_face)
{
  int		i;
  VECT3D	*A;
  VECT3D	*B;
  VECT3D	*C;
  VECT3D	*D;
  T_obj		*obj;
  T_face	*face;
  FILE	*fp;

  fp = fopen("toto", "w");
  
  A = (VECT3D *)malloc(sizeof(VECT3D));
  B = (VECT3D *)malloc(sizeof(VECT3D));
  C = (VECT3D *)malloc(sizeof(VECT3D));
  D = (VECT3D *)malloc(sizeof(VECT3D));
  obj = (T_obj *)malloc(sizeof(T_obj));
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));
  obj->name = "cone";
  obj->type_obj = OBJ_CONE;
  obj->prop->cone.center1 = c1;
  obj->prop->cone.radius1 = r1;
  obj->prop->cone.center2 = c2;
  obj->prop->cone.radius2 = r2;
  obj->prop->cone.detail_level = 25;
  obj->prop_base->list_vertex = NULL;
  obj->prop_base->nb_face = 0;

  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;
  
  if (!is_openC1)
    {
      A->x = c1->x;
      A->y = c1->y;
      A->z = c1->z;
      /* North pole */
      for (i = 0; i < obj->prop->cone.detail_level; i++)
	{      
	  B->x = obj->prop->cone.center1->x + 
	    obj->prop->cone.radius1 * cos(i * 2 * M_PI / 
					obj->prop->cone.detail_level);
	  B->y = obj->prop->cone.center1->y;
	  B->z = obj->prop->cone.center1->z + 
	    obj->prop->cone.radius1 * sin(i * 2 * M_PI /
					obj->prop->cone.detail_level);
	  C->x = obj->prop->cone.center1->x +
	    obj->prop->cone.radius1 * cos((i + 1) * 2 * M_PI /
					obj->prop->cone.detail_level);
	  C->y = obj->prop->cone.center1->y;
	  C->z = obj->prop->cone.center1->z + 
	    obj->prop->cone.radius1 * sin((i + 1) * 2 * M_PI /
					obj->prop->cone.detail_level);
      
	  face = make_face(A, B, C);
	  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, 
							 face);
	}
      obj->prop_base->nb_face += obj->prop->cone.detail_level;
    }
  /* the mid zone */
  for (i = 0; i < obj->prop->cone.detail_level; i++)
    {
      A->x = obj->prop->cone.center1->x +
	obj->prop->cone.radius1 * cos(i * 2 * M_PI /
				    obj->prop->cone.detail_level);
      A->y = obj->prop->cone.center1->y;
      A->z = obj->prop->cone.center1->z +
	obj->prop->cone.radius1 * sin(i * 2 * M_PI /
				    obj->prop->cone.detail_level);
      B->x = obj->prop->cone.center1->x +
	obj->prop->cone.radius1 * cos((i + 1) * 2 * M_PI /
				    obj->prop->cone.detail_level);
      B->y = obj->prop->cone.center1->y;
      B->z = obj->prop->cone.center1->z +
	obj->prop->cone.radius1 * sin((i + 1) * 2 * M_PI /
				    obj->prop->cone.detail_level);
      C->x = obj->prop->cone.center2->x +
	obj->prop->cone.radius2 * cos(i * 2 * M_PI /
				    obj->prop->cone.detail_level);
      C->y = obj->prop->cone.center2->y;
      C->z = obj->prop->cone.center2->z +
	obj->prop->cone.radius2 * sin(i * 2 * M_PI /
				    obj->prop->cone.detail_level);
      D->x = obj->prop->cone.center2->x +
	obj->prop->cone.radius2 * cos((i + 1) * 2 * M_PI /
				    obj->prop->cone.detail_level);
      D->y = obj->prop->cone.center2->y;
      D->z = obj->prop->cone.center2->z +
	obj->prop->cone.radius2 * sin((i + 1) * 2 * M_PI /
				    obj->prop->cone.detail_level);
      
      face = make_face(B, A, C);
      obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						     face);
      face = make_face(B, C, D);
      obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						     face);
      if (is_double_face)
	{
	  face = make_face(B, C, A);
	  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						     face);
	  face = make_face(B, D, C);
	  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex,
						     face);
	}
    }
  if (!is_openC2)
    {
      /* and the south pole */
      A->x = obj->prop->cone.center2->x;
      A->y = obj->prop->cone.center2->y;
      A->z = obj->prop->cone.center2->z;
      for (i = 0; i < obj->prop->cone.detail_level; i++)
	{
	  B->x = obj->prop->cone.center2->x + 
	    obj->prop->cone.radius2 * cos(i * 2 * M_PI / 
					obj->prop->cone.detail_level);
	  B->y = obj->prop->cone.center2->y;
	  B->z = obj->prop->cone.center2->z + 
	    obj->prop->cone.radius2 * sin(i * 2 * M_PI /
					obj->prop->cone.detail_level);
	  C->x = obj->prop->cone.center2->x +
	    obj->prop->cone.radius2 * cos((i + 1) * 2 * M_PI /
					obj->prop->cone.detail_level);
	  C->y = obj->prop->cone.center2->y;

	  C->z = obj->prop->cone.center2->z + 
	    obj->prop->cone.radius2 * sin((i + 1) * 2 * M_PI /
					obj->prop->cone.detail_level);
	  face = make_face(B, A, C);
      fprintf(fp, "%f\t%f\t%f\n", face->A->x, face->A->y, face->A->z);
      fprintf(fp, "%f\t%f\t%f\n", face->B->x, face->B->y, face->B->z);
      fprintf(fp, "%f\t%f\t%f\n", face->C->x, face->C->y, face->C->z);
	  
      obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, 
							 face);
	}
      obj->prop_base->nb_face += obj->prop->cone.detail_level;
    }
  obj->prop_base->nb_face += obj->prop->cone.detail_level * 2;
  if (is_double_face)
    obj->prop_base->nb_face += obj->prop->cone.detail_level * 2;
  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;
      obj->prop_base->color[i].y = 0.5;
      obj->prop_base->color[i].z = 0.5;
    }
  fprintf(fp, "\n\n\n\n\n");
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      face = ReadTriangleOfMesh(obj->prop_base->list_vertex, i);
      fprintf(fp, "%f\t%f\t%f\n", face->A->x, face->A->y, face->A->z);
      fprintf(fp, "%f\t%f\t%f\n", face->B->x, face->B->y, face->B->z);
      fprintf(fp, "%f\t%f\t%f\n", face->C->x, face->C->y, face->C->z);
    }
  obj->prop_base->is_selected = 0;
  free(A);
  free(B);
  free(C);
  free(D);
  return obj;
}


T_obj *make_MeshTriangleTorus(VECT3D *center, float radius, 
			     float vradius)
{
  int		i, j;
  VECT3D	*A;
  VECT3D	*B;
  VECT3D	*C;
  VECT3D	*D;
  T_obj		*obj;
  T_face	*face;
  float		*t_cos, *t_sin;
    
  /* init basics proporties of torus */
  obj = (T_obj *)malloc(sizeof(T_obj));
  obj->name = "torus";
  obj->type_obj = OBJ_TORUS;
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->prop->torus.center = center;
  obj->prop->torus.radius = radius;
  obj->prop->torus.vradius = vradius;
  obj->prop->torus.detail_level = 25;
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));
  obj->prop_base->list_vertex = NULL;
  A = (VECT3D *)malloc(sizeof(VECT3D));
  B = (VECT3D *)malloc(sizeof(VECT3D));
  C = (VECT3D *)malloc(sizeof(VECT3D));
  D = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;

  t_cos = (float *)malloc(sizeof(float) * obj->prop->torus.detail_level + 1);
  t_sin = (float *)malloc(sizeof(float) * obj->prop->torus.detail_level + 1);
  for (i = 0; i <= obj->prop->torus.detail_level; i++)
    {
      t_cos[i] = cos(i * 2 * M_PI / obj->prop->torus.detail_level);
      t_sin[i] = sin(i * 2 * M_PI / obj->prop->torus.detail_level);
    }

  for (i = 0; i < obj->prop->torus.detail_level; i++)
    for (j = 0; j < obj->prop->torus.detail_level; j++)
      {
	A->x = obj->prop->torus.center->x +
	  obj->prop->torus.radius * t_cos[i] + 
	  obj->prop->torus.vradius* t_cos[i] * t_cos[j];
	A->y = obj->prop->torus.center->y + obj->prop->torus.vradius * t_sin[j];
	A->z = obj->prop->torus.center->z + obj->prop->torus.radius * t_sin[i] +
	  obj->prop->torus.vradius* t_sin[i] * t_cos[j];
	
	B->x = obj->prop->torus.center->x + obj->prop->torus.radius * t_cos[i+1] +
	  obj->prop->torus.vradius* t_cos[i+1] * t_cos[j];
	B->y = obj->prop->torus.center->y + obj->prop->torus.vradius * t_sin[j];
	B->z = obj->prop->torus.center->z + obj->prop->torus.radius * t_sin[i+1] +
	  obj->prop->torus.vradius* t_sin[i+1] * t_cos[j];

	C->x = obj->prop->torus.center->x + obj->prop->torus.radius * t_cos[i] +
	  obj->prop->torus.vradius* t_cos[i] * t_cos[j+1];
	C->y = obj->prop->torus.center->y + obj->prop->torus.vradius * t_sin[j+1];
	C->z = obj->prop->torus.center->z + obj->prop->torus.radius * t_sin[i] +
	  obj->prop->torus.vradius* t_sin[i] * t_cos[j+1];
	
	D->x = obj->prop->torus.center->x + obj->prop->torus.radius * t_cos[i+1] +
	  obj->prop->torus.vradius* t_cos[i+1] * t_cos[j+1];
	D->y = obj->prop->torus.center->y + obj->prop->torus.vradius * t_sin[j+1];
	D->z = obj->prop->torus.center->z + obj->prop->torus.radius * t_sin[i+1] +
	  obj->prop->torus.vradius* t_sin[i+1] * t_cos[j+1];
	
	face = make_face(B, A, C);
	obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
	face = make_face(B, C, D);
	obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
      }
  obj->prop_base->nb_face = 2 * obj->prop->torus.detail_level * obj->prop->torus.detail_level;
  free(t_cos);
  free(t_sin);

  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  /* set color object */
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;
      obj->prop_base->color[i].y = 0.5;
      obj->prop_base->color[i].z = 0.5;
    }
  obj->prop_base->is_selected = 0;
  free(A);
  free(B);
  free(C);
  free(D);
  return obj;
}

T_obj *make_MeshTrianglePlane(VECT3D *B1, VECT3D *P)
{
  T_obj		*obj;
  VECT3D	*A;
  VECT3D	*B;
  VECT3D	*C;
  VECT3D	*D;
  T_face	*face;
  VECT3D	B2, B3;	/* (B1, B2, B3) is an orthogonal base */
  int		size_of_plane = 50;
  int		i;

  obj = (T_obj *)malloc(sizeof(T_obj));
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));  
  /* init basics proporties of torus */
  obj->name = "plane";
  obj->type_obj = OBJ_PLANE;
  obj->prop->plane.N = B1;
  obj->prop->plane.P = P;
  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;


  obj->prop_base->list_vertex = NULL;
  A = (VECT3D *)malloc(sizeof(VECT3D));
  B = (VECT3D *)malloc(sizeof(VECT3D));
  C = (VECT3D *)malloc(sizeof(VECT3D));
  D = (VECT3D *)malloc(sizeof(VECT3D));

  B2.x = B1->y;
  B2.y = -B1->x;
  B2.z =  0;
  B3 = CrossProduct(*B1, B2);
  A->x = P->x;
  A->y = P->y;
  A->z = P->z;
  B->x = P->x + size_of_plane * B2.x;
  B->y = P->y + size_of_plane * B2.y;
  B->z = P->z + size_of_plane * B2.z;
  C->x = P->x + size_of_plane * B2.x + size_of_plane * B3.x;
  C->y = P->y + size_of_plane * B2.y + size_of_plane * B3.y;
  C->z = P->z + size_of_plane * B2.z + size_of_plane * B3.z;
  D->x = P->x + size_of_plane * B3.x;
  D->y = P->y + size_of_plane * B3.y;
  D->z = P->z + size_of_plane * B3.z;
  face = make_face(A, B, C);
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(A, C, B);
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  face = make_face(A, C, D);
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);  
  face = make_face(A, D, C);
  obj->prop_base->list_vertex = AddTriangleToMesh(obj->prop_base->list_vertex, face);
  obj->prop_base->nb_face = 4;
  
  /* set color object */
  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;
      obj->prop_base->color[i].y = 0.5;
      obj->prop_base->color[i].z = 0.5;
    }
  obj->prop_base->is_selected = 0;
  free(A);
  free(B);
  free(C);
  free(D);
  return obj;
}

T_obj *make_MeshTriangleSor(GList *list, int nb_pt)
{
  T_obj		*obj;
  T_obj		*tmp_obj;
  GList		*current;
  T_pt_sor	*pt1;
  T_pt_sor	*pt2;
  int		i;
  VECT3D	*C1;	/* the first point 1*/
  VECT3D	*C2;	/* the seconf point 2 */
  
  obj = (T_obj *)malloc(sizeof(T_obj));
  obj->prop_base = (ObjectBase *)malloc(sizeof(ObjectBase));
  obj->prop = (T_prop_ *)malloc(sizeof(T_prop_));
  obj->name = "sor";
  obj->type_obj = OBJ_SOR;
  obj->prop->sor.detail_level = 25;
  obj->prop_base->list_vertex = NULL;
  obj->prop_base->nb_face = 0;
  obj->prop_base->rotate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->translate = (VECT3D *)malloc(sizeof(VECT3D));
  obj->prop_base->scale = (VECT3D *)malloc(sizeof(VECT3D));
  memset(obj->prop_base->rotate, 0, sizeof(VECT3D));
  memset(obj->prop_base->translate, 0, sizeof(VECT3D));
  obj->prop_base->scale->x = 1;
  obj->prop_base->scale->y = 1;
  obj->prop_base->scale->z = 1;

  C1 = (VECT3D *)malloc(sizeof(VECT3D));
  C2 = (VECT3D *)malloc(sizeof(VECT3D));
  for (i = 0; i < nb_pt - 1; i++)
    {
      current  = g_list_nth(list, i);
      pt1 = (T_pt_sor *)current->data;
      current  = g_list_nth(list, i + 1);
      pt2 = (T_pt_sor *)current->data;
      C1->x = 0;
      C1->y = pt1->y;
      C1->z = 0;
      C2->x = 0;
      C2->y = pt2->y;
      C2->z = 0;
      if (i == 0)
	tmp_obj = make_MeshTriangleCone(C1, pt1->x, 0, C2, pt2->x, 1, 0);
      else
	{
	  if (i == nb_pt - 2)
	    tmp_obj = make_MeshTriangleCone(C1, pt1->x, 1, C2, pt2->x, 0, 0);
	  else
	    tmp_obj = make_MeshTriangleCone(C1, pt1->x, 1, C2, pt2->x, 1, 0);
	}
      obj->prop_base->list_vertex = g_slist_concat(obj->prop_base->list_vertex, 
						 tmp_obj->prop_base->list_vertex);
      obj->prop_base->nb_face += tmp_obj->prop_base->nb_face;
    }
  printf("%d\n", obj->prop_base->nb_face);
  /* set color object */
  
  obj->prop_base->normal = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  obj->prop_base->color = (VECT3D *)malloc(sizeof(VECT3D) * obj->prop_base->nb_face);
  for (i = 0; i < obj->prop_base->nb_face; i++)
    {
      obj->prop_base->color[i].x = 0.5;
      obj->prop_base->color[i].y = 0.5;
      obj->prop_base->color[i].z = 0.5;
    }
  obj->prop_base->is_selected = 0;
  return obj;
}
