#include <glib.h>
#ifndef MLTYPES
#define MLTYPES
#include "../../mltypes.h"
#endif
typedef enum _T_axis
{
  AXIS_X,
  AXIS_Y,
  AXIS_Z
} T_axis;

typedef enum  _T_alter_type
{
  ALT_ROTATE, // rotation of the object
  ALT_TRANSLATE, // translation
  ALT_SCALE  // scaling
} T_alter_type;

typedef enum _T_relation_type
{
  REL_NOOP, // no operations, selecting a son will select the son
  REL_UNION, // father is union of sons
  REL_MERGE, // father is merging of sons
  REL_INTER, // father is intersection of sons
  REL_DIFF, // father is son1-son2
  REL_GROUP // no operations, but selecting a son will select father
} T_relation_type;

typedef enum _T_node_type
{
  NOD_NONE, //nothing at all
  NOD_SCENE, // root of the tree, unique
  NOD_LIGHT, // lightsource
  NOD_PLANE, // infinite plane
  NOD_CUBE, // cube
  NOD_SPHERE, // ...
  NOD_TORUS,
  NOD_CONE,
  NOD_CYLINDRE,
  NOD_SOR,
  NOD_CAMERA, // point of view, unique
  NOD_CSG, // composite object
  NOD_TRANS
} T_node_type;

typedef struct  // member of an alteration chain
{
  T_alter_type	type;
  VECT3D	*value;
} T_alter;

typedef struct	_T_cube_prop // properties of cube
{
  VECT3D	*point1;
  VECT3D	*point2;
} T_cube_prop;

typedef struct	_T_sphere_prop // properties of sphere
{
  VECT3D	*center;
  float		radius;
} T_sphere_prop;

typedef struct	_T_plane_prop // properties of plane
{
  VECT3D	*normal;
  float		offset;
} T_plane_prop;

typedef struct	_T_cone_prop // properties of cone
{
  VECT3D	*center1;
  float		radius1;
  VECT3D	*center2;
  float		radius2;
  int		open;
} T_cone_prop;

typedef struct  _T_camera_prop // properties of camera
{
  VECT3D	*location;
  VECT3D	*look_at;
  VECT3D	*sky;
  float		angle;
} T_camera_prop;

typedef struct 
{
  int num_point;
  GList *list;
} T_sor_prop;

typedef struct	_T_light_prop  // properties of lightsource
{
  VECT3D	*location;
} T_light_prop;

typedef struct	_T_scene_prop  //properties of scene
{
  gchar		*name;
  gchar		*author;
  gchar		*date;
  gchar		*filename;
} T_scene_prop;

typedef struct
{
  float		major;
  float		minor;
} T_torus_prop;

typedef union	_T_prop
{
  T_cube_prop	cube;
  T_sphere_prop	sphere;
  T_plane_prop	plane;
  T_cone_prop	cone;
  T_light_prop	light;
  T_scene_prop	scene;
  T_sor_prop	sor;
  T_torus_prop	torus;
  T_camera_prop	camera;
} T_prop;

typedef union	_T_color
{
  gchar		*name;
  VECT3D	*rgb;
} T_color;

typedef struct	_T_texture
{
  gchar		*name;
  T_color	*color;
  T_alter	*trans;
} T_texture;

typedef struct _T_filter
{
  int		color;
  int		value; // WARNING: this value must be between 0 and 100 and divided by 100 
} T_filter;

typedef struct		_T_node
{
  char			*name;
  //  UID unsigned int
  T_alter		*trans;
  T_node_type		type;
  T_color		*color;
  T_texture		*texture;
  T_prop		prop;
  // struct _T_node	*son1;
  // struct _T_node	*son2;
  T_relation_type	relation;
} T_node;

int node_count (T_node *dad, T_node_type type);



