#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"
#include "utils.h"
#include "object.h"

xmlNodePtr xml_camera_newprop(T_camera_prop *prop)
{
  xmlNodePtr tree;
  
  tree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(tree,"location",(char *)vect3d2str(prop->location));
  xmlSetProp(tree,"look_at",(char *)vect3d2str(prop->look_at));
  xmlSetProp(tree,"sky",(char *)vect3d2str(prop->sky));
  xmlSetProp(tree,"angle",g_strdup_printf("%f",prop->angle));

  return tree;

}

xmlNodePtr xml_camera_new(VECT3D *location, VECT3D *look_at,
			  VECT3D *sky, float angle)
{
  xmlNodePtr tree,subtree;

  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-camera");
  xmlSetProp(tree,"type","camera");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"location",(char *)vect3d2str(location));
  xmlSetProp(subtree,"look_at",(char *)vect3d2str(look_at));
  xmlSetProp(subtree,"sky",(char *)vect3d2str(sky));
  xmlSetProp(subtree,"angle",g_strdup_printf("%f", angle));
  xmlAddChild(tree,subtree);
  return tree;
}

xmlNodePtr xml_camera_new_char(char *location, char *look_at,
			       char *sky, char *angle)
{
  xmlNodePtr tree,subtree;

  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-camera");
  xmlSetProp(tree,"type","camera");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"location", location);
  xmlSetProp(subtree,"look_at", look_at);
  xmlSetProp(subtree,"sky", sky);
  xmlSetProp(subtree,"angle", angle);
  xmlAddChild(tree,subtree);
  return tree;
}

VECT3D *xml_camera_get_location(xmlNodePtr tree)
{
  xmlNodePtr tree2=xml_object_select_private(tree);

  return str2vect3d(xmlGetProp(tree,"location"));
}

VECT3D *xml_camera_get_look_at(xmlNodePtr tree)
{
  xmlNodePtr tree2=xml_object_select_private(tree);

  return str2vect3d(xmlGetProp(tree,"look_at"));
}

VECT3D *xml_camera_get_sky(xmlNodePtr tree)
{
  xmlNodePtr tree2=xml_object_select_private(tree);

  return str2vect3d(xmlGetProp(tree2,"sky"));
}

float xml_camera_get_angle(xmlNodePtr tree)
{
  xmlNodePtr tree2=xml_object_select_private(tree);

  return atof(xmlGetProp(tree2,"angle"));  
}

T_prop *xml_camera_get_prop(xmlNodePtr tree)
{
  T_prop *prop=(T_prop *)g_malloc(sizeof(T_prop));

  prop->camera.location=str2vect3d(xmlGetProp(tree,"location"));
  prop->camera.look_at=str2vect3d(xmlGetProp(tree,"look_at"));
  prop->camera.sky=str2vect3d(xmlGetProp(tree,"sky"));
  prop->camera.angle=atof(xmlGetProp(tree,"angle"));

  return prop;
}














