#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"
#include "utils.h"
#include "object.h"


xmlNodePtr xml_cone_newprop(T_cone_prop *prop)
{
  xmlNodePtr tree;
  

  puts("xml_cone_newprop");
  tree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(tree,"center1", 
	     (char *)vect3d2str(prop->center1));
  xmlSetProp(tree,"radius1", 
	     g_strdup_printf("%f", prop->radius1));
  xmlSetProp(tree,"center2", 
	     (char *)vect3d2str(prop->center2));
  xmlSetProp(tree,"radius2", 
	     g_strdup_printf("%f", prop->radius2));
  xmlSetProp(tree,"open",
	     g_strdup_printf("%d", prop->open));
  

  return tree;

}

xmlNodePtr xml_cone_new(VECT3D *center1, float radius1,
			VECT3D *center2, float radius2, int open)
{
  xmlNodePtr tree,subtree;
  
  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-cone");
  xmlSetProp(tree,"type","cone");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"center1", (char *)vect3d2str(center1));
  xmlSetProp(subtree,"radius1", g_strdup_printf("%f",radius1));
  xmlSetProp(subtree,"center2", (char *)vect3d2str(center2));
  xmlSetProp(subtree,"radius2", g_strdup_printf("%f",radius2));
  xmlSetProp(subtree,"open", g_strdup_printf("%d",open));
  xmlAddChild(tree,subtree);
  return tree;
}

xmlNodePtr xml_cone_new_char(char *center1, char *radius1,
			     char *center2, char *radius2, int open)
{
  xmlNodePtr tree,subtree;

  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-cone");
  xmlSetProp(tree,"type","cone");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"center1", center1);
  xmlSetProp(subtree,"radius1", radius1);
  xmlSetProp(subtree,"center2", center2);
  xmlSetProp(subtree,"radius2", radius2);
  if (open)
    xmlSetProp(subtree,"open", "open");
  else
    xmlSetProp(subtree,"open","");
  xmlAddChild(tree,subtree);

  return tree;
}

VECT3D *xml_cone_get_center1(xmlNodePtr tree)
{
  return str2vect3d(xmlGetProp(tree->childs,"center1"));
}

xmlNodePtr xml_cone_set_center1(xmlNodePtr tree, VECT3D *center1)
{
  xmlSetProp(tree,"center1", (char *)vect3d2str(center1));
  
  return tree;
}

float xml_cone_get_radius1(xmlNodePtr tree)
{
  return atof(xmlGetProp(tree->childs,"radius1"));
}

xmlNodePtr xml_cone_set_radius1(xmlNodePtr tree, float radius1)
{
  xmlSetProp(tree,"radius1", g_strdup_printf("%f",radius1));
  
  return tree;
}

VECT3D *xml_cone_get_center2(xmlNodePtr tree)
{
  return  str2vect3d(xmlGetProp(tree->childs,"center2"));
}

xmlNodePtr xml_cone_set_center2(xmlNodePtr tree, VECT3D *center2)
{
  xmlSetProp(tree,"center2", (char *)vect3d2str(center2));
  
  return tree;
}

float xml_cone_get_radius2(xmlNodePtr tree)
{
  return atof(xmlGetProp(tree->childs,"radius2"));
}

xmlNodePtr xml_cone_set_radius2(xmlNodePtr tree, float radius2)
{
  xmlSetProp(tree,"radius2", g_strdup_printf("%f",radius2));
  
  return tree;
}

int xml_cone_get_open(xmlNodePtr tree)
{
  return atoi(xmlGetProp(tree->childs,"open"));
}

xmlNodePtr xml_cone_set_open(xmlNodePtr tree, int open)
{
  xmlSetProp(tree,"open", g_strdup_printf("%d",open));

  return tree;
}

T_prop *xml_cone_get_prop(xmlNodePtr tree)
{
  T_prop *prop=(T_prop *)g_malloc(sizeof(T_prop));
  

  puts("xml_cone_get_prop");
  prop->cone.center1=str2vect3d(xmlGetProp(tree,"center1"));
  prop->cone.radius1=atof(xmlGetProp(tree,"radius1"));
  prop->cone.center2=str2vect3d(xmlGetProp(tree,"center2"));
  prop->cone.radius2=atof(xmlGetProp(tree,"radius2"));

  return prop;
}

void xml_cone_freeprop(T_cone_prop *prop)
{
#ifdef DEBUG
  if (prop!=NULL)
    printf("***DEBUG*** Cone properties free'd: %s\n",prop->name); 
#endif
  g_free(prop->center1);
  g_free(prop->center2);
  g_free(prop);
}
