#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"
#include "utils.h"
#include "object.h"

xmlNodePtr xml_cube_newprop(T_cube_prop *prop)
{
  xmlNodePtr tree;
  
  tree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(tree,"point1",(char *)vect3d2str(prop->point1));
  xmlSetProp(tree,"point2",(char *)vect3d2str(prop->point2));

  return tree;
}

xmlNodePtr xml_cube_new(VECT3D *point1, VECT3D *point2)
{
  xmlNodePtr tree,subtree;
  
  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-cube");
  xmlSetProp(tree,"type","box");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"point1",(char *)vect3d2str(point1));
  xmlSetProp(subtree,"point2",(char *)vect3d2str(point2));
  xmlAddChild(tree,subtree);

  return tree;
}

xmlNodePtr xml_cube_new_char(char *point1, char *point2)
{
  xmlNodePtr tree,subtree;

  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-cube");
  xmlSetProp(tree,"type","box");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"point1", point1);
  xmlSetProp(subtree,"point2", point2);
  xmlAddChild(tree,subtree);
  return tree;
}

VECT3D *xml_cube_get_point1(xmlNodePtr tree)
{
  return str2vect3d(xmlGetProp(tree->childs,"point1"));
}

xmlNodePtr xml_cube_set_point1(xmlNodePtr tree, VECT3D *point1)
{
  xmlSetProp(tree,"point1",(char *)vect3d2str(point1));

  return tree;
}

VECT3D *xml_cube_get_point2(xmlNodePtr tree)
{
  return str2vect3d(xmlGetProp(tree->childs,"point2"));
}

xmlNodePtr xml_cube_set_point2(xmlNodePtr tree, VECT3D *point2)
{
  xmlSetProp(tree,"point2",(char *)vect3d2str(point2));

  return tree;
}

T_prop *xml_cube_get_prop(xmlNodePtr tree)
{
  T_prop *prop=(T_prop *)g_malloc(sizeof(T_prop));

  prop->cube.point1=str2vect3d(xmlGetProp(tree,"point1"));
  prop->cube.point2=str2vect3d(xmlGetProp(tree,"point2"));

  return prop;
}

void xml_cube_freeprop(T_cube_prop *prop)
{
#ifdef DEBUG
  if (prop!=NULL)
    printf ("*** DEBUG *** Cube prop free'd: %s\n", prop->name);
#endif
  g_free(prop->point1);
  g_free(prop->point2);
  g_free(prop);
}
