#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"
#include "cube.h"
#include "utils.h"
#include "sphere.h"
#include "cone.h"
#include "plane.h"
#include "sor.h"
#include "torus.h"
#include "camera.h"
#include "csg.h"
#include "light.h"
#include "color.h"


xmlNodePtr xml_object_set_id(xmlNodePtr tree, uint id)
{
  xmlSetProp(tree,"ID",g_strdup_printf("%d",id));
    
  return tree;
}

uint xml_object_get_id(xmlNodePtr tree)
{
  if (tree==NULL) return 0;
  else return atoi(xmlGetProp(tree,"ID"));
}

xmlNodePtr xml_object_private_new(T_node *prop, T_node_type type)
{
  xmlNodePtr tree;
  
  switch (type)
    {
    case NOD_LIGHT:
      tree=xml_light_newprop(&prop->prop.light);
      break;
    case NOD_CAMERA:
      tree=xml_camera_newprop(&prop->prop.camera);
      break;
    case NOD_CONE:
      tree=xml_cone_newprop(&prop->prop.cone);
      break;
    case NOD_CUBE:
      tree=xml_cube_newprop(&prop->prop.cube);
      break;
    case NOD_SPHERE:
      tree=xml_sphere_newprop(&prop->prop.sphere);
      break;
    case NOD_PLANE:
      tree=xml_plane_newprop(&prop->prop.plane);
      break;
    case NOD_SOR:
      tree=xml_sor_newprop(&prop->prop.sor);
      break;
    case NOD_TORUS:
      tree=xml_torus_newprop(&prop->prop.torus);
      break;
    case NOD_CSG:
      tree=xml_csg_new(prop->name,prop->relation);
    default:
      return xmlNewNode(NULL,"#ERROR");
    }
  return tree;
}

xmlNodePtr xml_object_select_private(xmlNodePtr tree)
{
  xmlNodePtr tree2;

  if (tree->childs!=NULL)
    tree2=tree->childs;
  while (tree2->next!=NULL)
    {
      if (!strcmp(tree2->name,"PRIVATE"))
	{
	  return tree2;
	}
      tree2=tree2->next;
   }
  return NULL;
}

xmlNodePtr xml_object_select_texture(xmlNodePtr tree)
{
  xmlNodePtr tree2;

  if (tree->childs!=NULL)
    tree2=tree->childs;
  while (tree2->next!=NULL)
    {
      if (!strcmp(tree2->name,"TEXTURE"))
	{
	  return tree2;
	}
      tree2=tree2->next;
   }
  return NULL;
}

xmlNodePtr xml_object_select_color(xmlNodePtr tree)
{
  xmlNodePtr tree2;

  if (tree->childs!=NULL)
    tree2=tree->childs;
  while (tree2->next!=NULL)
    {
      if (!strcmp(tree2->name,"COLOR"))
	{
	  return tree2;
	}
      tree2=tree2->next;
   }
  return NULL;
}

T_prop *xml_object_get_private(xmlNodePtr tree)
{
  T_prop *private;
  xmlNodePtr tree2;
  T_node_type type=str2nodetype(xmlGetProp(tree,"type"));

  if (tree->childs!=NULL)
    tree2=tree->childs;
  while (tree2->next!=NULL)
    {
      if (!strcmp(tree2->name,"PRIVATE"))
	{
	  switch (type)
	    {
	    case NOD_LIGHT:
	      return xml_light_get_prop(tree);
	    case NOD_CUBE:
	      return xml_cube_get_prop(tree);
	    case NOD_SPHERE:
	      return xml_sphere_get_prop(tree);
	    case NOD_PLANE:
	      return xml_plane_get_prop(tree);
	    case NOD_CONE:
	      return xml_cone_get_prop(tree);
	    case NOD_TORUS:
	      return xml_torus_get_prop(tree);
	    case NOD_SOR:
	      return xml_sor_get_prop(tree);
	    case NOD_CAMERA:
	      return xml_camera_get_prop(tree);
	    case NOD_CSG:
	      return NULL;
	    default:
	      return NULL;
	    }
	  tree2=tree2->next;
	}
    }
}

xmlNodePtr xml_object_new(T_node *obj)
{
  xmlNodePtr tree,subtree;

  if (obj==NULL) return NULL;
  
  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name",obj->name);
  xmlSetProp(tree,"type",(const char *)nodetype2str(obj->type));
  if (obj->type==NOD_CSG)
    xmlSetProp(tree,"relation",(const char *)reltype2str(obj->relation));
     
  subtree=xml_object_private_new(obj,obj->type);
  
  xmlAddChild(tree,subtree);

  return tree;
}

char *xml_object_get_name(xmlNodePtr tree)
{
  return xmlGetProp(tree,"name");
}

T_node_type xml_object_get_type(xmlNodePtr tree)
{
  return str2nodetype(xmlGetProp(tree,"type"));
}

T_relation_type xml_object_get_relation(xmlNodePtr tree)
{
  return str2reltype(xmlGetProp(tree,"relation"));
}


