#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"

xmlDocPtr xml_scene_new(char *name, char *author, char *date, char *filename)
{
  xmlDocPtr doc;

  doc = xmlNewDoc("1.0");
  doc->root = xmlNewDocNode(doc, NULL, "MODLIGHT-SCENE", NULL);
  if (name != NULL)
    xmlSetProp(doc->root, "name", name);
  else
    xmlSetProp(doc->root, "name", "");
  if (author != NULL)
    xmlSetProp(doc->root, "author", author);
  else
    xmlSetProp(doc->root, "author", getenv("LOGNAME")); 
  if (date != NULL)
    xmlSetProp(doc->root, "date", date);
  else
    xmlSetProp(doc->root, "date", "0/0/00");
  if (filename != NULL)
    xmlSetProp(doc->root, "filename", filename);  
  else
    xmlSetProp(doc->root, "filename", "modlight-default.xml");
  return doc;
}

xmlDocPtr xml_scene_snew(T_scene_prop *prop)
{
  return xml_scene_new(prop->name,prop->author,prop->date,prop->filename);
}

char *xml_scene_get_name(xmlDocPtr doc)
{
  return xmlGetProp(xmlDocGetRootElement(doc),"name");
}

int xml_scene_set_name(xmlDocPtr doc, char *value)
{
  if (value!=NULL)
    xmlSetProp(xmlDocGetRootElement(doc),"name",value);
  else
    return -1;
}

char *xml_scene_get_author(xmlDocPtr doc)
{
  return xmlGetProp(xmlDocGetRootElement(doc),"author");
}


int xml_scene_set_author(xmlDocPtr doc, char *value)
{
  if (value!=NULL)
    xmlSetProp(xmlDocGetRootElement(doc),"author",value);
  else
    return -1;
}

char *xml_scene_get_date(xmlDocPtr doc)
{
  return xmlGetProp(xmlDocGetRootElement(doc),"date");
}

int xml_scene_set_date(xmlDocPtr doc, char *value)
{
  if (value!=NULL)
    xmlSetProp(xmlDocGetRootElement(doc),"date",value);
  else
    return -1;
}

char *xml_scene_get_filename(xmlDocPtr doc)
{
  return xmlGetProp(xmlDocGetRootElement(doc),"filename");
}

int xml_scene_set_filename(xmlDocPtr doc, char *value)
{
  if (value!=NULL)
    xmlSetProp(xmlDocGetRootElement(doc),"filename",value);
  else
    return -1;
}

xmlDocPtr xml_scene_load(char *filename, int overwrite)
{
  xmlDocPtr doc;
  
  if (filename == NULL) return NULL;

  doc=xmlParseFile(filename);
  
  if (overwrite)
    xml_scene_set_filename(doc,filename);
		   
}

int xml_scene_save(xmlDocPtr doc, short level)
{
  xmlNodePtr root;
  char *name;

  xmlSetCompressMode(level);
  root=xmlDocGetRootElement(doc);
  if (strcmp((name=xmlGetProp(root,"filename")),""))
    return xmlSaveFile(name,doc);
  else
    return -1;
}








