#include <stdlib.h>
#include <glib.h>
#include <tree.h>
#include <parser.h>
#include "../types/tree.h"
#include "scene.h"
#include "utils.h"
#include "object.h"

xmlNodePtr xml_sphere_newprop(T_sphere_prop *prop)
{
  xmlNodePtr tree;
  
  tree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(tree,"center",(char *)vect3d2str(prop->center));  
  xmlSetProp(tree,"radius",g_strdup_printf("%f",prop->radius));

  return tree;
}

xmlNodePtr xml_sphere_new(VECT3D *center, float radius)
{
  xmlNodePtr tree,subtree;
  
  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-sphere");
  xmlSetProp(tree,"type","sphere");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"center",(char *)vect3d2str(center));  
  xmlSetProp(subtree,"radius",g_strdup_printf("%f",radius));
  xmlAddChild(tree,subtree);
  return tree;
}

xmlNodePtr xml_sphere_new_char(char *center, char *radius)
{
  xmlNodePtr tree,subtree;

  tree=xmlNewNode(NULL,"OBJECT");
  xmlSetProp(tree,"name","object-sphere");
  xmlSetProp(tree,"type","sphere");
  subtree=xmlNewNode(NULL,"PRIVATE");
  xmlSetProp(subtree,"center", center);  
  xmlSetProp(subtree,"radius", radius);
  xmlAddChild(tree,subtree);
  return tree;
}

VECT3D *xml_sphere_get_center(xmlNodePtr tree)
{
  return str2vect3d(xmlGetProp(tree->childs,"center"));
}

xmlNodePtr xml_sphere_set_center(xmlNodePtr tree, VECT3D *center)
{
  xmlSetProp(tree,"center",(char *)vect3d2str(center));  
  
  return tree;
}

float xml_sphere_get_radius(xmlNodePtr tree)
{
  return atof(xmlGetProp(tree->childs,"radius"));
}

xmlNodePtr xml_sphere_set_radius(xmlNodePtr tree, float radius)
{
  xmlSetProp(tree,"radius",g_strdup_printf("%f",radius));

  return tree;
}

T_prop *xml_sphere_get_prop(xmlNodePtr tree)
{
  T_prop *prop=(T_prop *)g_malloc(sizeof(T_prop));

  prop->sphere.center=str2vect3d(xmlGetProp(tree,"center"));
  prop->sphere.radius=atof(xmlGetProp(tree,"radius"));

  return prop;
}

void xml_sphere_freeprop(T_sphere_prop *prop)
{
#ifdef DEBUG
  if (prop!=NULL)
    printf ("*** DEBUG *** Sphere prop free'd: %s\n",prop->name);
#endif
  g_free(prop->center);
  g_free(prop);
}
