#include <glib.h>
#include <stdlib.h>
#include "../types/tree.h"

gchar *vect3d2str(VECT3D *vect)
{
  gchar *temp;
  temp=g_strdup_printf("%f,%f,%f",vect->x,vect->y,vect->z);
  return temp;
}

gchar *vect2d2str(VECT2D *vect)
{
  gchar *temp;
  temp=g_strdup_printf("%f,%f",vect->x,vect->y);
  return temp;
}

VECT3D *str2vect3d(const char *str)
{
  VECT3D *vect;
  char **tab;
  double x,y,z;

  if (str==NULL)
    return NULL;
  
  vect=(VECT3D *)malloc(sizeof(VECT3D));
  tab=g_strsplit(str,",",3);
  
  vect->x=atof(tab[0]);
  vect->y=atof(tab[1]);
  vect->z=atof(tab[2]);

  return vect;
}

VECT2D *str2vect2d(const char *str)
{
  VECT2D *vect;
  char **tab;
  double x,y,z;

  if (str==NULL)
    return NULL;
  
  vect=(VECT2D *)malloc(sizeof(VECT2D));
  tab=g_strsplit(str,",",2);
  
  vect->x=atof(tab[0]);
  vect->y=atof(tab[1]);

  return vect;
}

char *nodetype2str(T_node_type type)
{
  switch(type)
    {
    case NOD_CAMERA:
      return "camera";
    case NOD_NONE:
      return "NOD_NONE";
    case NOD_SCENE:
      return "scene";
    case NOD_LIGHT:
      return "light_source";
    case NOD_PLANE:
      return "plane";
    case NOD_CUBE:
      return "box";
    case NOD_SPHERE:
      return "sphere";
    case NOD_TORUS:
      return "torus";
    case NOD_CONE:
      return "cone";
    case NOD_CYLINDRE:
      return "cylindre";
    case NOD_SOR:
      return "sor";
    case NOD_CSG:
      return "NOD_CSG";
    case NOD_TRANS:
      return "NOD_TRANS";
    default:
      return "NOD_UNKNOWN";
    }
}

T_node_type str2nodetype(char *str)
{
  if (str==NULL) return -1;
  
  if (!strcmp(str,"NOD_NONE"))		return NOD_NONE;
  if (!strcmp(str,"camera"))		return NOD_CAMERA;
  if (!strcmp(str,"scene"))		return NOD_SCENE;
  if (!strcmp(str,"light_source"))	return NOD_LIGHT;
  if (!strcmp(str,"plane"))		return NOD_PLANE;
  if (!strcmp(str,"box"))		return NOD_CUBE;
  if (!strcmp(str,"sphere"))		return NOD_SPHERE;
  if (!strcmp(str,"torus"))		return NOD_TORUS;
  if (!strcmp(str,"cone"))		return NOD_CONE;
  if (!strcmp(str,"cylindre"))		return NOD_CYLINDRE;
  if (!strcmp(str,"sor"))		return NOD_SOR;
  if (!strcmp(str,"NOD_CSG"))		return NOD_CSG;
  if (!strcmp(str,"NOD_TRANS"))		return NOD_TRANS;
  return -1;
  
}


char *reltype2str(T_relation_type type)
{
  switch(type)
    {
    case REL_NOOP:
      return "REL_NOOP";
    case REL_UNION:
      return "union";
    case REL_MERGE:
      return "merge";
    case REL_INTER:
      return "intersection";
    case REL_DIFF:
      return "difference";
    case REL_GROUP:
      return "REL_GROUP";
    default:
      return "REL_UNKNOWN";
    }
}

T_relation_type str2reltype(char *str)
{
  if (str==NULL) return -1;
  
  if (!strcmp(str,"REL_NOOP")) return REL_NOOP;
  if (!strcmp(str,"union")) return REL_UNION;
  if (!strcmp(str,"merge")) return REL_MERGE;
  if (!strcmp(str,"intersection")) return REL_INTER;
  if (!strcmp(str,"difference")) return REL_DIFF;
  if (!strcmp(str,"REL_GROUP")) return REL_GROUP;
  if (!strcmp(str,"REL_UNKNOWN")) return -1;
  return -1;
  
}


char *alttype2str(T_alter_type type)
{
  switch(type)
    {
    case ALT_ROTATE:
      return "rotate";
    case ALT_TRANSLATE:
      return "translate";
    case ALT_SCALE:
      return "scale";
    default:
      return "ALT_UNKNOWN";
    }
}

T_alter_type str2alttype(char *str)
{
  if (str==NULL) return -1;
  
  if (!strcmp(str,"rotate")) return ALT_ROTATE;
  if (!strcmp(str,"translate")) return ALT_TRANSLATE;
  if (!strcmp(str,"scale")) return ALT_SCALE;
  return -1;
  
}
